"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.saveInstance = exports.restoreInstance = void 0;

var clearChildren = function clearChildren(node) {
  if (node) {
    while (node.firstChild) {
      node.removeChild(node.firstChild);
    }
  }
};

var getMapInstanceId = function getMapInstanceId(id) {
  return "google-map-".concat(id);
};

var getHiddenMapContainer = function getHiddenMapContainer(id) {
  var hiddenMapContainer = "hidden-container-".concat(id);
  var element = document.getElementById(hiddenMapContainer);

  if (!element) {
    element = document.createElement('div');
    element.id = hiddenMapContainer;
    element.style.display = 'none';
    document.body.appendChild(element);
  }

  return element;
};

var restoreInstance = function restoreInstance(_ref) {
  var id = _ref.id,
      zoom = _ref.zoom,
      center = _ref.center,
      mapContainerStyle = _ref.mapContainerStyle;
  var map = window[getMapInstanceId(id)];
  var hiddenContainer = getHiddenMapContainer(id);

  if (map && hiddenContainer.children.length === 1) {
    map.setZoom(zoom);
    map.setCenter(center);
    map.setOptions({
      style: mapContainerStyle
    });
    var mapContainer = document.getElementById(id);
    clearChildren(mapContainer);
    mapContainer.appendChild(hiddenContainer.children[0]);
    return map;
  }
};

exports.restoreInstance = restoreInstance;

var saveInstance = function saveInstance(id, map) {
  var hiddenContainer = getHiddenMapContainer(id);
  clearChildren(hiddenContainer);
  var mapContainer = document.getElementById(id);
  hiddenContainer.appendChild(mapContainer.children[0]);
  window[getMapInstanceId(id)] = map;
};

exports.saveInstance = saveInstance;